@@ -1,4 +1,4 @@
-from .json_include import build_json
+from .json_include import build_str
 import argparse
 
 def main():
@@ -6,7 +6,7 @@ def main():
     parser.add_argument("filepath", help="JSON file to build")
     args = parser.parse_args()
 
-    data = build_json(args.filepath)
+    data = build_str(args.filepath)
     print(data)
 
 if __name__ == "__main__":
